@@ -28,7 +28,7 @@ HCPC <- function (res, nb.clust = 0, consol = TRUE, iter.max = 10, min = 3,
 		intra <- rev(cumsum(rev(inert.gain)))
         quot = intra[min:(max)]/intra[(min - 1):(max - 1)] 
 		nb.clust = which.min(quot) + min -1
-# changement dans calcul annul. Mis dans la version 1.34  2016/04/12 (2 lignes changees)
+# changement dans calcul annule. Mis dans la version 1.34  2016/04/12 (2 lignes changees)
 #        quot = inert.gain[(min-1):(max-1)]/inert.gain[min:max] 
 #		nb.clust = which.max(quot) + min - 1
         return(list(res = res, tree = hc, nb.clust = nb.clust, 
@@ -99,22 +99,34 @@ HCPC <- function (res, nb.clust = 0, consol = TRUE, iter.max = 10, min = 3,
     cla <- NULL
 
     if (inherits(res, "PCA") | inherits(res, "MCA") | inherits(res,"MFA") | inherits(res, "HMFA") | inherits(res, "FAMD")) {
-	  if (kk<nrow(res$ind$coord)) res <- as.data.frame(res$ind$coord)
+	  if (kk<nrow(res$ind$coord)){
+	    res <- as.data.frame(res$ind$coord)
+		kk <- min(kk, nrow(unique(res$ind$coord)))
+      }
     }
     if (inherits(res, "CA")) {
 	  if (cluster.CA=="rows"){
-	    if (kk<nrow(res$row$coord)) res <- as.data.frame(sweep(res$row$coord,2,sqrt(res$eig[1:ncol(res$row$coord),1]),FUN="*"))
+	    if (kk<nrow(res$row$coord)) {
+		  res <- as.data.frame(sweep(res$row$coord,2,sqrt(res$eig[1:ncol(res$row$coord),1]),FUN="*"))
+		  kk <- min(kk, nrow(unique(res$row$coord)))
+        }
 	  } else {
-	    if (kk<nrow(res$col$coord)) res <- as.data.frame(sweep(res$col$coord,2,sqrt(res$eig[1:ncol(res$col$coord),1]),FUN="*"))
+	    if (kk<nrow(res$col$coord)){
+		  res <- as.data.frame(sweep(res$col$coord,2,sqrt(res$eig[1:ncol(res$col$coord),1]),FUN="*"))
+  		  kk <- min(kk, nrow(unique(res$col$coord)))
+        }
 	  }
     }
     if (is.data.frame(res)){
 	  res <-  res[,unlist(lapply(res,is.numeric)),drop=FALSE]
 ### AJOUT K-means
-	  if (kk<nrow(res)){
-	    cla <- kmeans(res,centers=kk,iter.max = 100, nstart = 4)
+      if (kk<nrow(res)) kk <- min(kk,nrow(unique(res)))
+	  if (kk <= nrow(res)){
+	    cla <- kmeans(res, centers=kk, iter.max = 100, nstart = 4)
         res <- PCA(cla$centers, row.w=cla$size, scale.unit = FALSE, ncp = Inf, graph = FALSE)
-      } else res <- PCA(res, scale.unit = FALSE, ncp = Inf, graph = FALSE)
+      } else {
+	    res <- PCA(res, scale.unit = FALSE, ncp = Inf, graph = FALSE)
+	  }
 ### Fin AJOUT K-means
 ##      res <- PCA(res, scale.unit = FALSE, ncp = Inf, graph = FALSE)
     }
@@ -163,6 +175,8 @@ HCPC <- function (res, nb.clust = 0, consol = TRUE, iter.max = 10, min = 3,
             if (nb.clust == 0 | nb.clust == 1) nb.clust <- -1
         }
         if ((nb.clust == 0) | (nb.clust == 1)) {
+			  print("Click on the graph to cut the tree")
+			  flush.console()
 #            if (!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))){
               plot(t$tree, hang = -1, main = "Click to cut the tree",  xlab = "", sub = "")
               abline(h = auto.haut, col = "black", lwd = 3)
@@ -190,7 +204,6 @@ HCPC <- function (res, nb.clust = 0, consol = TRUE, iter.max = 10, min = 3,
 	X = as.data.frame(t$res$ind$coord)
 	ordColo = unique(clust[t$tree$order])
 
-
     if (graph) {
 #    if ((graph)&!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) {
 #        rect <- rect.hclust(t$tree, h = y, border = seq(1, nb.clust, 1))
@@ -243,9 +256,9 @@ centers = res.consol$centers
 		## Add 2014-07-08
 		if (kk<Inf){
 		  clust <- clust[order(as.integer(names(clust)))]
-		  rr=clust[cla$cluster]
-		  names(rr)=names(cla$cluster)
-		  cla$cluster=as.factor(rr)
+		  rr <- clust[cla$cluster]
+		  names(rr) <- names(cla$cluster)
+		  cla$cluster <- as.factor(rr)
 		}
 ## fin ajout		
     }
@@ -286,7 +299,9 @@ if (kk<Inf){
   if (inherits(res.sauv, "data.frame")) data.clust <- data.clust[rownames(res.sauv),]
   if (vec) data.clust <- as.data.frame(data.clust[, -2])
 if (description){
-     if (!inherits(res.sauv, "CA")&!(vec)) desc.var <- catdes(data.clust, ncol(data.clust), proba = proba, row.w = res$call$row.w.init)
+     if (!inherits(res.sauv, "CA")&!(vec)){
+	   desc.var <- catdes(data.clust, ncol(data.clust), proba = proba, row.w = res.sauv$call$row.w.init)
+	 }
     else {
       if ((vec) | (is.null(res.sauv$call$quanti.sup)& is.null(res.sauv$call$quali.sup))) desc.var <- descfreq(data.clust[,-which(sapply(data.clust,is.factor))], data.clust[,ncol(data.clust)], proba = proba)
 	  else { 
